# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class BulkIdErrorReason(str, Enum):
    """
    BulkIdErrorReason
    """

    """
    allowed enum values
    """
    DUPLICATE = "duplicate"
    NO_PERMISSION = "no_permission"
    NOT_FOUND = "not_found"
    UNKNOWN = "unknown"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of BulkIdErrorReason from a JSON string"""
        return cls(json.loads(json_str))



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List
from typing import Optional, Set
from typing_extensions import Self


class ServerConfigDto(BaseModel):
    """
    ServerConfigDto
    """  # noqa: E501

    external_domain: StrictStr = Field(alias="externalDomain")
    is_initialized: StrictBool = Field(alias="isInitialized")
    is_onboarded: StrictBool = Field(alias="isOnboarded")
    login_page_message: StrictStr = Field(alias="loginPageMessage")
    maintenance_mode: StrictBool = Field(alias="maintenanceMode")
    map_dark_style_url: StrictStr = Field(alias="mapDarkStyleUrl")
    map_light_style_url: StrictStr = Field(alias="mapLightStyleUrl")
    oauth_button_text: StrictStr = Field(alias="oauthButtonText")
    public_users: StrictBool = Field(alias="publicUsers")
    trash_days: StrictInt = Field(alias="trashDays")
    user_delete_delay: StrictInt = Field(alias="userDeleteDelay")
    __properties: ClassVar[List[str]] = [
        "externalDomain",
        "isInitialized",
        "isOnboarded",
        "loginPageMessage",
        "maintenanceMode",
        "mapDarkStyleUrl",
        "mapLightStyleUrl",
        "oauthButtonText",
        "publicUsers",
        "trashDays",
        "userDeleteDelay",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ServerConfigDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ServerConfigDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "externalDomain": obj.get("externalDomain"),
                "isInitialized": obj.get("isInitialized"),
                "isOnboarded": obj.get("isOnboarded"),
                "loginPageMessage": obj.get("loginPageMessage"),
                "maintenanceMode": obj.get("maintenanceMode"),
                "mapDarkStyleUrl": obj.get("mapDarkStyleUrl"),
                "mapLightStyleUrl": obj.get("mapLightStyleUrl"),
                "oauthButtonText": obj.get("oauthButtonText"),
                "publicUsers": obj.get("publicUsers"),
                "trashDays": obj.get("trashDays"),
                "userDeleteDelay": obj.get("userDeleteDelay"),
            }
        )
        return _obj



from __future__ import annotations

from pathlib import Path
from typing import Any, Optional
from uuid import UUID

from pydantic import StrictStr

from immich.client.api.assets_api import AssetsApi
from immich.client.models.asset_media_size import AssetMediaSize
from immich.utils import download_file, resolve_output_filename


class AssetsApiWrapped(AssetsApi):
    async def download_asset_to_file(
        self,
        id: UUID,
        out_dir: Path,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        filename: Optional[str] = None,
        show_progress: bool = True,
        **kwargs: Any,
    ) -> Path:
        """
        Download an asset to a file.

        :param id: The asset ID.
        :param out_dir: The directory to write the asset to.
        :param key: Public share key (the last path segment of a public share URL, i.e. `/share/<key>`). When provided, the asset can be accessed via the public share link without an API key. Typically you pass either `key` or `slug`.
        :param slug: Public share slug for custom share URLs (the last path segment of `/s/<slug>`). Allows access without authentication. Typically you pass either `slug` or `key`.
        :param filename: The filename to use. If not provided, we use the original filename from the headers or default to "orig-" + asset_id.
        :param show_progress: Whether to show a tqdm progress bar while downloading.
        :param kwargs: Additional arguments to pass to the `download_asset_without_preload_content` method.

        For exact request/response behavior, inspect `AssetsApi.download_asset_without_preload_content`
        in the generated client.
        """
        out_dir.mkdir(parents=True, exist_ok=True)

        def make_request(extra_headers: Optional[dict[str, str]]):
            return self.download_asset_without_preload_content(
                id=id,
                key=key,
                slug=slug,
                _headers=kwargs.get("_headers", {}) | (extra_headers or {}),
                **kwargs,
            )

        return await download_file(
            make_request=make_request,
            out_dir=out_dir,
            resolve_filename=lambda headers: resolve_output_filename(
                headers,
                name=filename,
                default_base=f"orig-{id}",
            ),
            show_progress=show_progress,
        )

    async def play_asset_video_to_file(
        self,
        id: UUID,
        out_dir: Path,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        filename: Optional[str] = None,
        show_progress: bool = True,
        **kwargs: Any,
    ) -> Path:
        """
        Save an asset's video stream to a file.

        :param id: The asset ID.
        :param out_dir: The directory to write the video to.
        :param key: Public share key (the last path segment of a public share URL, i.e. `/share/<key>`). When provided, the asset can be accessed via the public share link without an API key. Typically you pass either `key` or `slug`.
        :param slug: Public share slug for custom share URLs (the last path segment of `/s/<slug>`). Allows access without authentication. Typically you pass either `slug` or `key`.
        :param filename: The filename to use. If not provided, we use the original filename from the headers or default to "video-" + asset_id.
        :param show_progress: Whether to show a tqdm progress bar while downloading.
        :param kwargs: Additional arguments to pass to the `play_asset_video_without_preload_content` method.

        For exact request/response behavior, inspect `AssetsApi.play_asset_video_without_preload_content`
        in the generated client.
        """
        out_dir.mkdir(parents=True, exist_ok=True)

        def make_request(extra_headers: Optional[dict[str, str]]):
            return self.play_asset_video_without_preload_content(
                id=id,
                key=key,
                slug=slug,
                _headers=kwargs.get("_headers", {}) | (extra_headers or {}),
                **kwargs,
            )

        return await download_file(
            make_request=make_request,
            out_dir=out_dir,
            resolve_filename=lambda headers: resolve_output_filename(
                headers,
                name=filename,
                default_base=f"video-{id}",
            ),
            show_progress=show_progress,
        )

    async def play_asset_video_to_file(
        self,
        id: UUID,
        out_dir: Path,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        filename: Optional[str] = None,
        **kwargs: Any,
    ) -> Path:
        """
        Save an asset's video stream to a file.

        :param id: The asset ID.
        :param out_dir: The directory to write the video to.
        :param key: Public share key (the last path segment of a public share URL, i.e. `/share/<key>`). When provided, the asset can be accessed via the public share link without an API key. Typically you pass either `key` or `slug`.
        :param slug: Public share slug for custom share URLs (the last path segment of `/s/<slug>`). Allows access without authentication. Typically you pass either `slug` or `key`.
        :param filename: The filename to use. If not provided, we use the original filename from the headers or default to "video-" + asset_id.
        :param kwargs: Additional arguments to pass to the `play_asset_video_with_http_info` method.

        For exact request/response behavior, inspect `AssetsApi.play_asset_video_with_http_info`
        in the generated client.
        """
        resp = await super().play_asset_video_with_http_info(
            id=id, key=key, slug=slug, **kwargs
        )
        name = resolve_output_filename(
            resp.headers,
            name=filename,
            default_base=f"video-{id}",
        )

        out_dir.mkdir(parents=True, exist_ok=True)
        out_path = out_dir / name
        out_path.write_bytes(bytes(resp.data))
        return out_path

    async def view_asset_to_file(
        self,
        id: UUID,
        out_dir: Path,
        key: Optional[StrictStr] = None,
        size: Optional[AssetMediaSize] = None,
        slug: Optional[StrictStr] = None,
        filename: Optional[str] = None,
        show_progress: bool = True,
        **kwargs: Any,
    ) -> Path:
        """
        Save an asset's thumbnail to a file.

        :param id: The asset ID.
        :param out_dir: The directory to write the asset to.
        :param key: Public share key (the last path segment of a public share URL, i.e. `/share/<key>`). When provided, the thumbnail can be fetched via the public share link without an API key. Typically you pass either `key` or `slug`.
        :param size: Thumbnail size.
        :param slug: Public share slug for custom share URLs (the last path segment of `/s/<slug>`). Allows access without authentication. Typically you pass either `slug` or `key`.
        :param filename: The filename to use. If not provided, we use the original filename from the headers or default to "thumb-" + asset_id.
        :param show_progress: Whether to show a tqdm progress bar while downloading.
        :param kwargs: Additional arguments to pass to the `view_asset_without_preload_content` method.

        For exact request/response behavior, inspect `AssetsApi.view_asset_without_preload_content`
        in the generated client.
        """
        out_dir.mkdir(parents=True, exist_ok=True)

        def make_request(extra_headers: Optional[dict[str, str]]):
            return self.view_asset_without_preload_content(
                id=id,
                key=key,
                size=size,
                slug=slug,
                _headers=kwargs.get("_headers", {}) | (extra_headers or {}),
                **kwargs,
            )

        return await download_file(
            make_request=make_request,
            out_dir=out_dir,
            resolve_filename=lambda headers: resolve_output_filename(
                headers,
                name=filename,
                default_base=f"thumb-{id}",
            ),
            show_progress=show_progress,
        )



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Set
from typing_extensions import Self


class ExifResponseDto(BaseModel):
    """
    ExifResponseDto
    """  # noqa: E501

    city: Optional[StrictStr] = None
    country: Optional[StrictStr] = None
    date_time_original: Optional[datetime] = Field(
        default=None, alias="dateTimeOriginal"
    )
    description: Optional[StrictStr] = None
    exif_image_height: Optional[Union[StrictFloat, StrictInt]] = Field(
        default=None, alias="exifImageHeight"
    )
    exif_image_width: Optional[Union[StrictFloat, StrictInt]] = Field(
        default=None, alias="exifImageWidth"
    )
    exposure_time: Optional[StrictStr] = Field(default=None, alias="exposureTime")
    f_number: Optional[Union[StrictFloat, StrictInt]] = Field(
        default=None, alias="fNumber"
    )
    file_size_in_byte: Optional[StrictInt] = Field(default=None, alias="fileSizeInByte")
    focal_length: Optional[Union[StrictFloat, StrictInt]] = Field(
        default=None, alias="focalLength"
    )
    iso: Optional[Union[StrictFloat, StrictInt]] = None
    latitude: Optional[Union[StrictFloat, StrictInt]] = None
    lens_model: Optional[StrictStr] = Field(default=None, alias="lensModel")
    longitude: Optional[Union[StrictFloat, StrictInt]] = None
    make: Optional[StrictStr] = None
    model: Optional[StrictStr] = None
    modify_date: Optional[datetime] = Field(default=None, alias="modifyDate")
    orientation: Optional[StrictStr] = None
    projection_type: Optional[StrictStr] = Field(default=None, alias="projectionType")
    rating: Optional[Union[StrictFloat, StrictInt]] = None
    state: Optional[StrictStr] = None
    time_zone: Optional[StrictStr] = Field(default=None, alias="timeZone")
    __properties: ClassVar[List[str]] = [
        "city",
        "country",
        "dateTimeOriginal",
        "description",
        "exifImageHeight",
        "exifImageWidth",
        "exposureTime",
        "fNumber",
        "fileSizeInByte",
        "focalLength",
        "iso",
        "latitude",
        "lensModel",
        "longitude",
        "make",
        "model",
        "modifyDate",
        "orientation",
        "projectionType",
        "rating",
        "state",
        "timeZone",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ExifResponseDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if city (nullable) is None
        # and model_fields_set contains the field
        if self.city is None and "city" in self.model_fields_set:
            _dict["city"] = None

        # set to None if country (nullable) is None
        # and model_fields_set contains the field
        if self.country is None and "country" in self.model_fields_set:
            _dict["country"] = None

        # set to None if date_time_original (nullable) is None
        # and model_fields_set contains the field
        if (
            self.date_time_original is None
            and "date_time_original" in self.model_fields_set
        ):
            _dict["dateTimeOriginal"] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict["description"] = None

        # set to None if exif_image_height (nullable) is None
        # and model_fields_set contains the field
        if (
            self.exif_image_height is None
            and "exif_image_height" in self.model_fields_set
        ):
            _dict["exifImageHeight"] = None

        # set to None if exif_image_width (nullable) is None
        # and model_fields_set contains the field
        if (
            self.exif_image_width is None
            and "exif_image_width" in self.model_fields_set
        ):
            _dict["exifImageWidth"] = None

        # set to None if exposure_time (nullable) is None
        # and model_fields_set contains the field
        if self.exposure_time is None and "exposure_time" in self.model_fields_set:
            _dict["exposureTime"] = None

        # set to None if f_number (nullable) is None
        # and model_fields_set contains the field
        if self.f_number is None and "f_number" in self.model_fields_set:
            _dict["fNumber"] = None

        # set to None if file_size_in_byte (nullable) is None
        # and model_fields_set contains the field
        if (
            self.file_size_in_byte is None
            and "file_size_in_byte" in self.model_fields_set
        ):
            _dict["fileSizeInByte"] = None

        # set to None if focal_length (nullable) is None
        # and model_fields_set contains the field
        if self.focal_length is None and "focal_length" in self.model_fields_set:
            _dict["focalLength"] = None

        # set to None if iso (nullable) is None
        # and model_fields_set contains the field
        if self.iso is None and "iso" in self.model_fields_set:
            _dict["iso"] = None

        # set to None if latitude (nullable) is None
        # and model_fields_set contains the field
        if self.latitude is None and "latitude" in self.model_fields_set:
            _dict["latitude"] = None

        # set to None if lens_model (nullable) is None
        # and model_fields_set contains the field
        if self.lens_model is None and "lens_model" in self.model_fields_set:
            _dict["lensModel"] = None

        # set to None if longitude (nullable) is None
        # and model_fields_set contains the field
        if self.longitude is None and "longitude" in self.model_fields_set:
            _dict["longitude"] = None

        # set to None if make (nullable) is None
        # and model_fields_set contains the field
        if self.make is None and "make" in self.model_fields_set:
            _dict["make"] = None

        # set to None if model (nullable) is None
        # and model_fields_set contains the field
        if self.model is None and "model" in self.model_fields_set:
            _dict["model"] = None

        # set to None if modify_date (nullable) is None
        # and model_fields_set contains the field
        if self.modify_date is None and "modify_date" in self.model_fields_set:
            _dict["modifyDate"] = None

        # set to None if orientation (nullable) is None
        # and model_fields_set contains the field
        if self.orientation is None and "orientation" in self.model_fields_set:
            _dict["orientation"] = None

        # set to None if projection_type (nullable) is None
        # and model_fields_set contains the field
        if self.projection_type is None and "projection_type" in self.model_fields_set:
            _dict["projectionType"] = None

        # set to None if rating (nullable) is None
        # and model_fields_set contains the field
        if self.rating is None and "rating" in self.model_fields_set:
            _dict["rating"] = None

        # set to None if state (nullable) is None
        # and model_fields_set contains the field
        if self.state is None and "state" in self.model_fields_set:
            _dict["state"] = None

        # set to None if time_zone (nullable) is None
        # and model_fields_set contains the field
        if self.time_zone is None and "time_zone" in self.model_fields_set:
            _dict["timeZone"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ExifResponseDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "city": obj.get("city"),
                "country": obj.get("country"),
                "dateTimeOriginal": obj.get("dateTimeOriginal"),
                "description": obj.get("description"),
                "exifImageHeight": obj.get("exifImageHeight"),
                "exifImageWidth": obj.get("exifImageWidth"),
                "exposureTime": obj.get("exposureTime"),
                "fNumber": obj.get("fNumber"),
                "fileSizeInByte": obj.get("fileSizeInByte"),
                "focalLength": obj.get("focalLength"),
                "iso": obj.get("iso"),
                "latitude": obj.get("latitude"),
                "lensModel": obj.get("lensModel"),
                "longitude": obj.get("longitude"),
                "make": obj.get("make"),
                "model": obj.get("model"),
                "modifyDate": obj.get("modifyDate"),
                "orientation": obj.get("orientation"),
                "projectionType": obj.get("projectionType"),
                "rating": obj.get("rating"),
                "state": obj.get("state"),
                "timeZone": obj.get("timeZone"),
            }
        )
        return _obj



from __future__ import annotations

from typing import Optional

from aiohttp import ClientSession  # type: ignore[import-not-found]

from immich.client.api_client import ApiClient
from immich.client.configuration import Configuration

from immich.client.api.activities_api import ActivitiesApi
from immich.client.api.albums_api import AlbumsApi
from immich.client.api.api_keys_api import APIKeysApi
from immich.client_wrapper.assets_api_wrapped import AssetsApiWrapped
from immich.client.api.authentication_admin_api import AuthenticationAdminApi
from immich.client.api.authentication_api import AuthenticationApi
from immich.client.api.deprecated_api import DeprecatedApi
from immich.client_wrapper.download_api_wrapped import DownloadApiWrapped
from immich.client.api.duplicates_api import DuplicatesApi
from immich.client.api.faces_api import FacesApi
from immich.client.api.jobs_api import JobsApi
from immich.client.api.libraries_api import LibrariesApi
from immich.client.api.maintenance_admin_api import MaintenanceAdminApi
from immich.client.api.map_api import MapApi
from immich.client.api.memories_api import MemoriesApi
from immich.client.api.notifications_admin_api import NotificationsAdminApi
from immich.client.api.notifications_api import NotificationsApi
from immich.client.api.partners_api import PartnersApi
from immich.client.api.people_api import PeopleApi
from immich.client.api.plugins_api import PluginsApi
from immich.client.api.queues_api import QueuesApi
from immich.client.api.search_api import SearchApi
from immich.client.api.server_api import ServerApi
from immich.client.api.sessions_api import SessionsApi
from immich.client.api.shared_links_api import SharedLinksApi
from immich.client.api.stacks_api import StacksApi
from immich.client.api.sync_api import SyncApi
from immich.client.api.system_config_api import SystemConfigApi
from immich.client.api.system_metadata_api import SystemMetadataApi
from immich.client.api.tags_api import TagsApi
from immich.client.api.timeline_api import TimelineApi
from immich.client.api.trash_api import TrashApi
from immich.client.api.users_admin_api import UsersAdminApi
from immich.client_wrapper.users_api_wrapped import UsersApiWrapped
from immich.client.api.views_api import ViewsApi
from immich.client.api.workflows_api import WorkflowsApi


def _normalize_base_url(base_url: str) -> str:
    return base_url.rstrip("/")


def _build_configuration(
    *,
    api_key: Optional[str],
    bearer_token: Optional[str],
    cookie: Optional[str],
    base_url: str,
) -> Configuration:
    config = Configuration(host=_normalize_base_url(base_url))
    if api_key:
        # Security scheme name is `api_key` (header: x-api-key)
        config.api_key["api_key"] = api_key
    if cookie:
        # Security scheme name is `cookie` (cookie: immich_access_token)
        config.api_key["cookie"] = cookie
    if bearer_token:
        # JWT access token (Authorization: Bearer <token>)
        config.access_token = bearer_token
    return config


class AsyncClient:
    """
    Async client for the Immich API.
    """

    activities: ActivitiesApi
    """An activity is a like or a comment made by a user on an asset or album.

    https://api.immich.app/endpoints/activities
    """

    albums: AlbumsApi
    """An album is a collection of assets that can be shared with other users or via shared links.

    https://api.immich.app/endpoints/albums
    """

    api_keys: APIKeysApi
    """An api key can be used to programmatically access the Immich API.

    https://api.immich.app/endpoints/api-keys
    """

    assets: AssetsApiWrapped
    """An asset is an image or video that has been uploaded to Immich.

    https://api.immich.app/endpoints/assets
    """

    authentication: AuthenticationApi
    """Endpoints related to user authentication, including OAuth.

    https://api.immich.app/endpoints/authentication
    """

    authentication_admin: AuthenticationAdminApi
    """Administrative endpoints related to authentication.

    https://api.immich.app/endpoints/authentication-(admin)
    """

    deprecated: DeprecatedApi
    """Deprecated endpoints that are planned for removal in the next major release.

    https://api.immich.app/endpoints/deprecated
    """

    download: DownloadApiWrapped
    """Endpoints for downloading assets or collections of assets.

    https://api.immich.app/endpoints/download
    """

    duplicates: DuplicatesApi
    """Endpoints for managing and identifying duplicate assets.

    https://api.immich.app/endpoints/duplicates
    """

    faces: FacesApi
    """A face is a detected human face within an asset, which can be associated with a person. Faces are normally detected via machine learning, but can also be created via manually.

    https://api.immich.app/endpoints/faces
    """

    jobs: JobsApi
    """Queues and background jobs are used for processing tasks asynchronously. Queues can be paused and resumed as needed.

    https://api.immich.app/endpoints/jobs
    """

    libraries: LibrariesApi
    """An external library is made up of input file paths or expressions that are scanned for asset files. Discovered files are automatically imported. Assets much be unique within a library, but can be duplicated across libraries. Each user has a default upload library, and can have one or more external libraries.

    https://api.immich.app/endpoints/libraries
    """

    maintenance_admin: MaintenanceAdminApi
    """Maintenance mode allows you to put Immich in a read-only state to perform various operations.

    https://api.immich.app/endpoints/maintenance-(admin)
    """

    map: MapApi
    """Map endpoints include supplemental functionality related to geolocation, such as reverse geocoding and retrieving map markers for assets with geolocation data.

    https://api.immich.app/endpoints/map
    """

    memories: MemoriesApi
    """A memory is a specialized collection of assets with dedicated viewing implementations in the web and mobile clients. A memory includes fields related to visibility and are automatically generated per user via a background job.

    https://api.immich.app/endpoints/memories
    """

    notifications: NotificationsApi
    """A notification is a specialized message sent to users to inform them of important events. Currently, these notifications are only shown in the Immich web application.

    https://api.immich.app/endpoints/notifications
    """

    notifications_admin: NotificationsAdminApi
    """Notification administrative endpoints.

    https://api.immich.app/endpoints/notifications-(admin)
    """

    partners: PartnersApi
    """A partner is a link with another user that allows sharing of assets between two users.

    https://api.immich.app/endpoints/partners
    """

    people: PeopleApi
    """A person is a collection of faces, which can be favorited and named. A person can also be merged into another person. People are automatically created via the face recognition job.

    https://api.immich.app/endpoints/people
    """

    plugins: PluginsApi
    """A plugin is an installed module that makes filters and actions available for the workflow feature.

    https://api.immich.app/endpoints/plugins
    """

    queues: QueuesApi
    """Queues and background jobs are used for processing tasks asynchronously. Queues can be paused and resumed as needed.

    https://api.immich.app/endpoints/queues
    """

    search: SearchApi
    """Endpoints related to searching assets via text, smart search, optical character recognition (OCR), and other filters like person, album, and other metadata. Search endpoints usually support pagination and sorting.

    https://api.immich.app/endpoints/search
    """

    server: ServerApi
    """Information about the current server deployment, including version and build information, available features, supported media types, and more.

    https://api.immich.app/endpoints/server
    """

    sessions: SessionsApi
    """A session represents an authenticated login session for a user. Sessions also appear in the web application as "Authorized devices".

    https://api.immich.app/endpoints/sessions
    """

    shared_links: SharedLinksApi
    """A shared link is a public url that provides access to a specific album, asset, or collection of assets. A shared link can be protected with a password, include a specific slug, allow or disallow downloads, and optionally include an expiration date.

    https://api.immich.app/endpoints/shared-links
    """

    stacks: StacksApi
    """A stack is a group of related assets. One asset is the "primary" asset, and the rest are "child" assets. On the main timeline, stack parents are included by default, while child assets are hidden.

    https://api.immich.app/endpoints/stacks
    """

    sync: SyncApi
    """A collection of endpoints for the new mobile synchronization implementation.

    https://api.immich.app/endpoints/sync
    """

    system_config: SystemConfigApi
    """Endpoints to view, modify, and validate the system configuration settings.

    https://api.immich.app/endpoints/system-config
    """

    system_metadata: SystemMetadataApi
    """Endpoints to view, modify, and validate the system metadata, which includes information about things like admin onboarding status.

    https://api.immich.app/endpoints/system-metadata
    """

    tags: TagsApi
    """A tag is a user-defined label that can be applied to assets for organizational purposes. Tags can also be hierarchical, allowing for parent-child relationships between tags.

    https://api.immich.app/endpoints/tags
    """

    timeline: TimelineApi
    """Specialized endpoints related to the timeline implementation used in the web application. External applications or tools should not use or rely on these endpoints, as they are subject to change without notice.

    https://api.immich.app/endpoints/timeline
    """

    trash: TrashApi
    """Endpoints for managing the trash can, which includes assets that have been discarded. Items in the trash are automatically deleted after a configured amount of time.

    https://api.immich.app/endpoints/trash
    """

    users_admin: UsersAdminApi
    """Administrative endpoints for managing users, including creating, updating, deleting, and restoring users. Also includes endpoints for resetting passwords and PIN codes.

    https://api.immich.app/endpoints/users-(admin)
    """

    users: UsersApiWrapped
    """Endpoints for viewing and updating the current users, including product key information, profile picture data, onboarding progress, and more.

    https://api.immich.app/endpoints/users
    """

    views: ViewsApi
    """Endpoints for specialized views, such as the folder view.

    https://api.immich.app/endpoints/views
    """

    workflows: WorkflowsApi
    """A workflow is a set of actions that run whenever a triggering event occurs. Workflows also can include filters to further limit execution.

    https://api.immich.app/endpoints/workflows
    """

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        bearer_token: Optional[str] = None,
        cookie: Optional[str] = None,
        base_url: str,
        http_client: Optional[ClientSession] = None,
    ) -> None:
        self._owns_http_client = http_client is None
        self._injected_http_client = http_client
        self.config = _build_configuration(
            api_key=api_key,
            bearer_token=bearer_token,
            cookie=cookie,
            base_url=base_url,
        )
        self.base_client = ApiClient(configuration=self.config)
        self.base_client.user_agent = "immich-python-client"

        # Allow caller to inject a pre-configured aiohttp session.
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client

        # API groups (single upstream API, not microservices)
        self.activities = ActivitiesApi(self.base_client)
        self.albums = AlbumsApi(self.base_client)
        self.api_keys = APIKeysApi(self.base_client)
        self.assets = AssetsApiWrapped(self.base_client)
        self.authentication = AuthenticationApi(self.base_client)
        self.authentication_admin = AuthenticationAdminApi(self.base_client)
        self.deprecated = DeprecatedApi(self.base_client)
        self.download = DownloadApiWrapped(self.base_client)
        self.duplicates = DuplicatesApi(self.base_client)
        self.faces = FacesApi(self.base_client)
        self.jobs = JobsApi(self.base_client)
        self.libraries = LibrariesApi(self.base_client)
        self.maintenance_admin = MaintenanceAdminApi(self.base_client)
        self.map = MapApi(self.base_client)
        self.memories = MemoriesApi(self.base_client)
        self.notifications = NotificationsApi(self.base_client)
        self.notifications_admin = NotificationsAdminApi(self.base_client)
        self.partners = PartnersApi(self.base_client)
        self.people = PeopleApi(self.base_client)
        self.plugins = PluginsApi(self.base_client)
        self.queues = QueuesApi(self.base_client)
        self.search = SearchApi(self.base_client)
        self.server = ServerApi(self.base_client)
        self.sessions = SessionsApi(self.base_client)
        self.shared_links = SharedLinksApi(self.base_client)
        self.stacks = StacksApi(self.base_client)
        self.sync = SyncApi(self.base_client)
        self.system_config = SystemConfigApi(self.base_client)
        self.system_metadata = SystemMetadataApi(self.base_client)
        self.tags = TagsApi(self.base_client)
        self.timeline = TimelineApi(self.base_client)
        self.trash = TrashApi(self.base_client)
        self.users = UsersApiWrapped(self.base_client)
        self.users_admin = UsersAdminApi(self.base_client)
        self.views = ViewsApi(self.base_client)
        self.workflows = WorkflowsApi(self.base_client)

    async def close(self) -> None:
        # If the caller injected the aiohttp session, don't close it.
        if not self._owns_http_client and self._injected_http_client is not None:
            self.base_client.rest_client.pool_manager = None
        await self.base_client.close()

    async def __aenter__(self) -> "AsyncClient":
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        await self.close()



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt
from typing import Any, ClassVar, Dict, List
from immich.client.models.usage_by_user_dto import UsageByUserDto
from typing import Optional, Set
from typing_extensions import Self


class ServerStatsResponseDto(BaseModel):
    """
    ServerStatsResponseDto
    """  # noqa: E501

    photos: StrictInt
    usage: StrictInt
    usage_by_user: List[UsageByUserDto] = Field(alias="usageByUser")
    usage_photos: StrictInt = Field(alias="usagePhotos")
    usage_videos: StrictInt = Field(alias="usageVideos")
    videos: StrictInt
    __properties: ClassVar[List[str]] = [
        "photos",
        "usage",
        "usageByUser",
        "usagePhotos",
        "usageVideos",
        "videos",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ServerStatsResponseDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in usage_by_user (list)
        _items = []
        if self.usage_by_user:
            for _item_usage_by_user in self.usage_by_user:
                if _item_usage_by_user:
                    _items.append(_item_usage_by_user.to_dict())
            _dict["usageByUser"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ServerStatsResponseDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "photos": obj.get("photos") if obj.get("photos") is not None else 0,
                "usage": obj.get("usage") if obj.get("usage") is not None else 0,
                "usageByUser": [
                    UsageByUserDto.from_dict(_item) for _item in obj["usageByUser"]
                ]
                if obj.get("usageByUser") is not None
                else None,
                "usagePhotos": obj.get("usagePhotos")
                if obj.get("usagePhotos") is not None
                else 0,
                "usageVideos": obj.get("usageVideos")
                if obj.get("usageVideos") is not None
                else 0,
                "videos": obj.get("videos") if obj.get("videos") is not None else 0,
            }
        )
        return _obj
